set halfdroprate 0

proc printpkts { label tcp tfrc } {
      global tcpTick_ opt ns
      set time [expr [$ns now]]
      set numTicks [$tcp set rtt_]
      set rtt [expr $numTicks * $tcpTick_]
      # tcppkts: number of packets sent
      set tcppkts [$tcp set ndatapack_]
      # print packets and bytes for TCP flow.
      puts "time: [format "%.2f" $time] tcp $label pkts_sent $tcppkts pkts_acked [$tcp set ack_] bytes_sent [$tcp set ndatabytes_] rtt [format "%5.3f" $rtt]"
      # print loss event rates for TCP flow
      set lossEvents [$tcp set ncwndcuts1_]
      if {$tcppkts > 0} {
  	  set lossEventRate [expr (1.0*$lossEvents) / (1.0*$tcppkts)]
      } else {
      	  set lossEventRate 0
      }
      puts "time: [format "%.2f" $time] tcp $label cwndcuts $lossEvents lossEventRate [format "%.4f" $lossEventRate]" 
      # print loss event rates for TFRC flow
      if {$opt(tfrc) == 1} {
        set tfrcLosses [$tfrc set true_loss_rate_]
        set tfrcpkts [$tfrc set ndatapack_]
	set tfrcbytes [$tfrc set ndatabytes_]
        set headerbytes [expr $tfrcpkts * 32]
        #  print packets and bytes for TFRC flow.
        puts "time: [format "%.2f" $time] tfrc $label pkts_sent $tfrcpkts databytes $tfrcbytes headerbytes $headerbytes rtt [format "%5.3f" $rtt]"
        puts "time: [format "%.2f" $time] tfrc $label true_loss_rate $tfrcLosses"
      }
}

proc printtime {} { 
	global ns opt
	set time [expr [$ns now]]
        if !$opt(terse) {
          puts "time: [format "%.2f" $time]"
	}
}

proc printdrops { fid fmon } { 
	global opt ns 
        set fcl [$fmon classifier]; # flow classifier
        set flow [$fcl lookup auto 0 0 $fid]
	set time [expr [$ns now]]
	if {$flow != ""} {
	  set pdrops 0
	  set pdrops [$flow set pdrops_]
	  set marks 0
	  set marks [$flow set pmarks_]
          #puts "fid: $fid per-link total_drops $pdrops"
          #puts "fid: $fid per-link total_marks $marks"
	  set pkts 0
	  set pkts [$flow set pdepartures_]
	  set apkts [$flow set parrivals_]
          #puts "fid: $fid per-link total_packet_departures $pkts"
	  set bytes [$flow set bdepartures_]
	  set sendbytes [$flow set barrivals_]
          #puts "fid: $fid per-link total_byte_departures $bytes"
	  if {$apkts > 0} {
	  	set droprate [expr 1.0000 * ($pdrops + $marks) / $apkts]
       	  } else {
	      	set droprate 0
          }
	  if $opt(terse) {
	    puts -nonewline " [format "%.4f" $droprate]"
	  } else {
	     puts "fid: $fid per-link drop_rate [format "%.4f" $droprate] pkt_arrivals $apkts drops $pdrops pkt_departures $pkts"
	  }
	  set now [expr [$ns now]]
	  if $opt(terse) {
	    set answer [expr $bytes ]
	    #puts -nonewline " [format "%.4f" $answer]"
          } else {
	    #puts "fid: $fid per-link bytes $bytes"
	  }
	  if $opt(terse) {
	    set answer [expr $sendbytes ]
	    puts -nonewline " [format "%d" $answer]"
          } else {
	    puts "fid: $fid per-link bytes [format "%d" $sendbytes]"
	  }
       }
}


proc psprintdrops { fmon } {
    for {set i 0} {$i < 30000} {incr i} {
	printdrops $i $fmon
    }
}

proc plotQueue prologue {
    global quiet tchan_ out opt
	#
	# Plot the queue size and average queue size, for AQM gateways
	#
    set awkCode {
	{
	    if ($1 == "Q" && NF>2) {
		print $2, $3 >> "temp.q";
		set end $2
	    }
	}
    }
    set awkCode1 {
	{
	    if ($1 == "a" && NF>2) {
		print $2, $3 >> "temp1.q";
		set end $2
	    }
	}
    }

#temp.q contains instantaneous queue length, temp1.q contains average queue length; we will just plot instantaneous q.


    if { [info exists tchan_] } {
	close $tchan_
    }
    exec rm -f temp.q temp1.q temp2.q
    exec touch temp.q temp1.q temp2.q

# To plot the instantaneous queue:
    exec awk $awkCode all.q.$out
# To plot the average queue:
    exec awk $awkCode1 all.q.$out

}


proc printall { fmon stoptime rate now } {
    global squeue opt halfdroprate halfbytes halftime
    puts "time=$now"
    set drops [$fmon set pdrops_]
    set bytes [$fmon set bdepartures_]
    set pkts [$fmon set pdepartures_]
    puts "pkts $pkts, drops $drops, bytes $bytes, time $now"
    #puts "Q_weight [$squeue set q_weight_]"
    if {$pkts > 0} {
	set droprate [expr 100.0*$drops/$pkts ]
        puts [format "aggregate per-link drops(percent) %0.3f time $now" $droprate]
	if {$now == $halftime} {set halfdroprate $droprate}
	if {$now == $opt(stop)} {
           set secondhalfdroprate [expr 2.0 * $droprate - $halfdroprate]
	   puts [format "aggregate per-link drops(percent) second half %0.3f" $secondhalfdroprate]
	}
    } else {
       puts "aggregate per-link drops(%) 0 time $now"
    }
    puts [format "aggregate per-link throughput(percent) %0.3f time $now" $bytes]
    if {$now == $halftime} {set halfbytes $bytes}
    if {$now == $opt(stop)} {
	set secondhalfbytes [expr 2.0 * $bytes - $halfbytes]
 	puts [format "aggregate per-link throughput(percent) second half %0.3f" $secondhalfbytes]
    }
}
